#include "Aerobics.h"

Aerobics::Aerobics()
{
	last_down = 0;
	last_jump = 0;

	stretch_left_status = 0;
	stretch_right_status = 0;

	lift_left_status = 0;
	lift_right_status = 0;

	left_wait_time = 0;
	right_wait_time = 0;

	special_down = 0;
	last_special_down = 0;

	stretch_front_left_status = 0;
	stretch_front_right_status = 0;

	memset(result, -1, 4 * sizeof(int));

	memset(left_global_pos, 0, 3 * sizeof(int));
	memset(right_global_pos, 0, 3 * sizeof(int));

}

void Aerobics::setData(deque<shoes_data_cell>& shoes_data_vector, int time_stamp, float pos_x, float pos_y, float pos_z, float heading, float pitch, float roll, int zupt, int rssi)
{
	if (shoes_data_vector.size() > 0)
	{
		if (shoes_data_vector.front().zupt == 0)
		{
			shoes_data_vector.clear();
		}
		else if (shoes_data_vector.back().zupt == 1 && zupt == 1)
		{
			shoes_data_vector.clear();
		}
	}
	shoes_data_vector.push_back({ time_stamp,  pos_x,  pos_y,  pos_z, heading, pitch, roll, zupt, rssi });
}

void Aerobics::calGlobalPos(deque<shoes_data_cell>& shoes_data_vector, deque<shoes_data_cell>& step_data_vector, float* global_pos, int LEFT_OR_RIGHT)
{
	int length = shoes_data_vector.size();

	if (length == 0)
	{
		memset(global_pos, 0, 3 * sizeof(float));
		step_data_vector.clear();

		return;
	}

	shoes_data_cell shoes_data_temp = shoes_data_vector.back();

	if (length == 1)
	{
		step_data_vector.clear();

		shoes_data_temp.pos_x = 0;
		shoes_data_temp.pos_y = 0;
		shoes_data_temp.pos_z = 0;

		step_data_vector.push_back(shoes_data_temp);

		memset(global_pos, 0, 3 * sizeof(float));

		return;
	}

	float pos_offset[3];

	pos_offset[0] = shoes_data_vector.at(length - 1).pos_x - shoes_data_vector.at(length - 2).pos_x;
	pos_offset[1] = shoes_data_vector.at(length - 1).pos_y - shoes_data_vector.at(length - 2).pos_y;
	pos_offset[2] = shoes_data_vector.at(length - 1).pos_z - shoes_data_vector.at(length - 2).pos_z;

	float posTemp[3];

	float heading = shoes_data_vector[0].heading;

	posTemp[0] = cos(heading) * pos_offset[0] + sin(heading) * pos_offset[1];
	posTemp[1] = -sin(heading) * pos_offset[0] + cos(heading) * pos_offset[1];


	if (LEFT_OR_RIGHT == LEFT_FOOT)
	{
		/*
		*  ΪЬЬӳģ²򣬻ʹƵX켣䳤
		*/
		pos_offset[0] = 0.9848 * posTemp[0] - 0.1736 * posTemp[1];
		pos_offset[1] = 0.1736 * posTemp[0] + 0.9848 * posTemp[1];
	}
	else
	{
		pos_offset[0] = 0.9848 * posTemp[0] + 0.1736 * posTemp[1];
		pos_offset[1] = -0.1736 * posTemp[0] + 0.9848 * posTemp[1];
	}

	for (int i = 0; i < 3; i++)
	{
		global_pos[i] += pos_offset[i];
	}

	shoes_data_temp.pos_x = step_data_vector.back().pos_x + pos_offset[0];
	shoes_data_temp.pos_y = step_data_vector.back().pos_y + pos_offset[1];
	shoes_data_temp.pos_z = step_data_vector.back().pos_z + pos_offset[2];

	step_data_vector.push_back(shoes_data_temp);
}

void Aerobics::Process(int time_stamp, int* right_pos, int* right_att, int* right_acc, int right_zupt, int right_front_press, int right_back_press,
	int* left_pos, int* left_att, int* left_acc, int left_zupt, int left_front_press, int left_back_press,
	int jump, int down, int rssi)
{
	memset(result, -1, 4 * sizeof(int));

	setData(right_shoes_data_vector, time_stamp, right_pos[0] * 0.001f, right_pos[1] * 0.001f, right_pos[2] * 0.001f,
		right_att[0] * 0.0001f, right_att[1] * 0.0001f, right_att[2] * 0.0001f, right_zupt, rssi);

	setData(left_shoes_data_vector, time_stamp, left_pos[0] * 0.001f, left_pos[1] * 0.001f, left_pos[2] * 0.001f,
		left_att[0] * 0.0001f, left_att[1] * 0.0001f, left_att[2] * 0.0001f, left_zupt, rssi);

	calGlobalPos(right_shoes_data_vector, right_step_data_vector, right_global_pos, RIGHT_FOOT);

	calGlobalPos(left_shoes_data_vector, left_step_data_vector, left_global_pos, LEFT_FOOT);

	//õŵĿ״̬ʹڿֻжһΣڴزܴ

	if (left_zupt)
	{
		stretch_left_status = 1;

		stretch_front_left_status = 1;

		lift_left_status = 1;
	}

	if (right_zupt)
	{
		stretch_right_status = 1;

		stretch_front_right_status = 1;

		lift_right_status = 1;
	}

	if (left_step_data_vector.size() > 1 && left_step_data_vector.front().zupt == 1 && left_step_data_vector.back().zupt == 0)
	{
		//ͳƲ
		addMotionCount(STEP_COUNT);

		setResult(left_step_data_vector, LEFT_FOOT);
	}

	if (right_step_data_vector.size() > 1 && right_step_data_vector.front().zupt == 1 && right_step_data_vector.back().zupt == 0)
	{
		//ͳƲ
		addMotionCount(STEP_COUNT);

		setResult(right_step_data_vector, RIGHT_FOOT);
	}



	//ж
	if (last_down == 0 && down == 1)
	{
		//ͳƲ
		addMotionCount(DOWN_COUNT);

		std::cout << "''Ķ" << endl;
		result[0] = MOTION_DOWN;

	}
	
	//жڽŶ
	if (left_att[1] * 0.0001f > 0.1f && right_att[1] * 0.0001f > 0.1f && down == 1)
	{
		special_down = 1;

	}
	else
	{
		special_down = 0;
	}

	//if (special_down == 1 && last_special_down == 0)
	//{
	//	std::cout << "ڽ''Ķ" << endl;
	//	result[0] = MOTION_DOWN_FRONT;
	//}

	last_special_down = special_down;

	last_down = down;
}

void Aerobics::setResult(deque<shoes_data_cell>& step_data_vector,int LEFT_OR_RIGHT)
{
	// й켣
	//ʵʵʱ⵽켣ǷϴģΪ˷򵥣ڴص˲
	//1ȼ⵽ZĹ켣Ƿ10cm, ھͻᴥж
	//2 ͬʱͬһѭжƽ棨X,Yĵ

	float high = step_data_vector.back().pos_z;

	float plane_length = sqrt(step_data_vector.back().pos_x * step_data_vector.back().pos_x 
		+ step_data_vector.back().pos_y * step_data_vector.back().pos_y);

	float pitch = step_data_vector.back().pitch;

	float roll  = step_data_vector.back().roll;

	//ʱȽ̧ŵĸ߶ֵΪ0.1m, ŵľΪ0.2m
	if (high > high_thresh && plane_length > 0.2)
	{
		if (LEFT_OR_RIGHT == LEFT_FOOT && stretch_left_status == 1 && roll > -2.9f && roll < 0.0f)
		{
			std::cout << "Ŷ" << endl;
			result[0] = MOTION_STRETCH_LEFT;
			stretch_left_status = 0;
			lift_left_status = 0;
		}
		else if(LEFT_OR_RIGHT == RIGHT_FOOT && stretch_right_status == 1 && roll > 0.0f && roll < 2.9f)
		{
			std::cout << "Ŷ" << endl;
			result[0] = MOTION_STRETCH_RIGHT;
			stretch_right_status = 0;
			lift_right_status = 0;
		}

		if (LEFT_OR_RIGHT == LEFT_FOOT && stretch_front_left_status == 1
			&& step_data_vector.back().pos_y > 0.2f)
		{
			std::cout << "߽Ŷ" << endl;
			result[0] = MOTION_STRETCH_FRONT_LEFT;
			stretch_front_left_status = 0;
			lift_left_status = 0;
		}
		else if (LEFT_OR_RIGHT == RIGHT_FOOT && stretch_front_right_status == 1 
			&& step_data_vector.back().pos_y > 0.2f)
		{
			std::cout << "߽Ŷ" << endl;
			result[0] = MOTION_STRETCH_FRONT_RIGHT;
			stretch_front_right_status = 0;
			lift_right_status = 0;
		}

	}
	else if (high > high_thresh && plane_length < 0.2f)
	{
		if (LEFT_OR_RIGHT == LEFT_FOOT && lift_left_status == 1)
		{
			std::cout << "̧ȶ" << endl;
			result[0] = MOTION_LIFT_LEFT;
			lift_left_status = 0;
		}
		else if (LEFT_OR_RIGHT == RIGHT_FOOT && lift_right_status == 1)
		{
			std::cout << "̧ȶ" << endl;
			result[0] = MOTION_LIFT_RIGHT;
			lift_right_status = 0;
		}
	}
}



void Aerobics::getResult(int* dec)
{
	memcpy(dec, result, 4 * sizeof(int));
}

float Aerobics::getGamePos(int left_or_right, int index)
{
	if (index < 0 || index > 2)
		return -1;

	if (left_or_right == LEFT_FOOT)
	{
		return left_global_pos[index];
	}
	else
	{
		return right_global_pos[index];
	}
}